﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Termination {
  [Item("ExecutionTimeTimeTerminationCriterion", "")]
  [StorableClass]
  public class ExecutionTimeTimeTerminationCriterion : TerminationCriterion {

    private IValueParameter<TimeSpanValue> MaximumExecutionTimeParameter {
      get { return (IValueParameter<TimeSpanValue>)Parameters["MaximumExecutionTime"]; }
    }

    [Storable]
    private readonly IExecutable executable;

    [StorableConstructor]
    protected ExecutionTimeTimeTerminationCriterion(bool deserializing) : base(deserializing) { }
    protected ExecutionTimeTimeTerminationCriterion(ExecutionTimeTimeTerminationCriterion original, Cloner cloner)
      : base(original, cloner) {
      executable = cloner.Clone(executable);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExecutionTimeTimeTerminationCriterion(this, cloner);
    }
    public ExecutionTimeTimeTerminationCriterion(IExecutable executable)
      : this(executable, new TimeSpan(0, 10, 0)) { }
    public ExecutionTimeTimeTerminationCriterion(IExecutable executable, TimeSpan maximumExecutionTime) {
      this.executable = executable;
      Parameters.Add(new ValueParameter<TimeSpanValue>("MaximumExecutionTime", "The maximum execution time of an algortihm.", new TimeSpanValue(TimeSpan.FromMinutes(10))));
    }

    protected override bool CheckTermination() {
      var max = MaximumExecutionTimeParameter.Value.Value;

      return executable.ExecutionTime > max;
    }
  }
}